package com.jeeplus.test.finance.invoice.service.dto;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.jeeplus.core.domain.BaseEntity;
import com.jeeplus.core.service.dto.BaseDTO;
import com.jeeplus.test.oss.domain.WorkAttachment;
import com.jeeplus.test.workContract.service.dto.WorkAttachmentDto;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 财务管理-发票
 * @TableName finance_invoice
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class FinanceInvoiceDTO extends BaseDTO {

    public static final String BIZ_CODE = "4";

    /**
     * 备注信息
     */
    private String remarks;

    /**
     * 发票类型
     */
    @Excel(name = "发票类型", orderNum = "0", width = 30)
    private String type;

    /**
     * 发票申请编号
     */
    @Excel(name = "发票申请编号", orderNum = "0", width = 30)
    private String no;

    /**
     * 开票类型
     */
    @Excel(name = "开票类型", orderNum = "0", width = 30)
    private String billingType;

    /**
     * 实际开票单位
     */
    @Excel(name = "实际开票单位", orderNum = "0", width = 30)
    private String billingWorkplaceReal;

    /**
     * 纳税人识别号
     */
    private String taxpayerIdentificationNo;

    /**
     * 地址
     */
    private String address;

    /**
     * 电话
     */
    private String telPhone;

    /**
     * 开户银行
     */
    private String openBank;

    /**
     * 银行账号
     */
    private String bankAccount;

    /**
     * 姓名
     */
    private String name;

    /**
     * 收款类型
     */
    @Excel(name = "收款类型", orderNum = "0", width = 30)
    private String receivablesType;

    /**
     * 开票内容
     */
    @Excel(name = "开票内容", orderNum = "0", width = 30)
    private String billingContent;

    /**
     * 发票金额(元)
     */
    @Excel(name = "发票金额(元)", orderNum = "0", width = 30)
    private String account;

    /**
     * 开票内容要求
     */
    private String billingContentTerms;

    /**
     * 开票人
     */
    private String billingPeople;

    /**
     * 开票时间
     */
    @Excel(name = "开票时间", orderNum = "0", width = 30)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date billingDate;

    /**
     * 领票时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date collectDate;

    /**
     * 实际开票人
     */
    private String billingPeopleReal;

    /**
     * 对账人id
     */
    private String reconciliationPeople;

    /**
     * 对账人姓名
     */
    private String reconciliationPeopleName;

    /**
     * 对账地区
     */
    private String reconciliationArea;

    /**
     * 实际开票单位id
     */
    private String billingWorkplaceRealId;

    /**
     * 开票人id
     */
    private String billingPeopleId;

    /**
     * 状态
     */
    @Excel(name = "状态", orderNum = "0", width = 30)
    private String status;

    /**
     * 流程id
     */
    private String procInsId;

    /**
     * 流程信息
     */
    private String processDefinitionId;

    /**
     * 是否确认收款
     */
    @Excel(name = "是否确认收款", orderNum = "0", width = 30)
    private String receivablesStatus;

    /**
     * 是否作废
     */
    @Excel(name = "是否作废", orderNum = "0", width = 30)
    private String invalidStatus;

    /**
     * 收款日期
     */
    @Excel(name = "收款日期", orderNum = "0", width = 30)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date receivablesDate;

    /**
     * 基础信息
     */
    private List<FinanceInvoiceBaseDTO> financeInvoiceBaseDTOList;

    /**
     * 附件信息
     */
    private List<WorkAttachmentDto> workAttachmentDtoList;

    /**
     * 发票明细
     */
    private List<FinanceInvoiceDetailDTO> financeInvoiceDetailDTOList;

    /**
     * 收款明细
     */
    private List<FinanceInvoiceReceivablesDTO> financeInvoiceReceivablesDTOList;

    /**
     * 作废信息
     */
    private FinanceInvoiceInvalidDTO financeInvoiceInvalidDTO;

    /**
     * 发票号
     */
    @Excel(name = "发票号", orderNum = "0", width = 30)
    private String number;

    /**
     * 开票总金额 Begin
     */
    private String accountBegin;

    /**
     * 开票总金额 End
     */
    private String accountEnd;

    /**
     * 开票日期 Begin
     */
    private String billingDateBegin;

    /**
     * 开票日期 End
     */
    private String billingDateEnd;

    /**
     * 收款日期Begin
     */
    private String remittanceDateBegin;

    /**
     * 收款日期End
     */
    private String remittanceDateEnd;

    /**
     * 项目名称
     */
    @Excel(name = "项目名称", orderNum = "0", width = 30)
    private String programName;

    /**
     * 项目id
     */
    private String programId;

    /**
     * 经办人
     */
    @Excel(name = "经办人", orderNum = "0", width = 30)
    private String operator;

    /**
     * 经办人部门
     */
    @Excel(name = "经办人部门", orderNum = "0", width = 30)
    private String operatorOffice;

    private static final long serialVersionUID = 1L;
}